/*
	Sudoku generator

	(C) Copyright 2005 by Mogens Kjaer, mk@lemo.dk

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

extern int nfilled,newnfilled,bestndigits,ndigits;
extern int symmetrical;
extern int level;

extern int displayboard[9][9];

extern int setboard[9][9];
extern int setboardsave[9][9];
extern int beforeboard[9][9];
static int possible[9][9];
extern int board[9][9];
extern int nmove,nsol,ntest,running,solved;

static int randomize=1;
static int nmultiple,nsccell,nscnumber,nssc,nds;
static int fastalg=0;

int printout=0;
void logprintf(const char *template, ...);

static void move();

/* calculate the number of bits set to 1 in p: */
#define NBITS(p) ({int n; n = (p); n = (0x5555 & n) + (0x5555 & (n>> 1)); n = (0x3333 & n) + (0x3333 & (n>> 2)); n = (0x0f0f & n) + (0x0f0f & (n>> 4)); n = (0x00ff & n) + (0x00ff & (n>> 8)); n;})
/*
	The following random number generator
	code is taken from glibc, containing
	the following copyright:

*/


/*
   Copyright (C) 1995, 2005 Free Software Foundation

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

/*
   Copyright (C) 1983 Regents of the University of California.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
   4. Neither the name of the University nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.*/

/*
 * This is derived from the Berkeley source:
 *	@(#)random.c	5.5 (Berkeley) 7/6/88
 * It was reworked for the GNU C Library by Roland McGrath.
 * Rewritten to be reentrant by Ulrich Drepper, 1995
 */

struct local_random_data
  {
    int *fptr;		/* Front pointer.  */
    int *rptr;		/* Rear pointer.  */
    int *state;		/* Array of state values.  */
    int rand_type;		/* Type of random number generator.  */
    int rand_deg;		/* Degree of random number generator.  */
    int rand_sep;		/* Distance between front and rear.  */
    int *end_ptr;		/* Pointer behind state table.  */
  };

static int randtbl[32] =
  {
    3,

    -1726662223, 379960547, 1735697613, 1040273694, 1313901226,
    1627687941, -179304937, -2073333483, 1780058412, -1989503057,
    -615974602, 344556628, 939512070, -1249116260, 1507946756,
    -812545463, 154635395, 1388815473, -1926676823, 525320961,
    -1009028674, 968117788, -123449607, 1284210865, 435012392,
    -2017506339, -911064859, -370259173, 1132637927, 1398500161,
    -205601318,
  };


static struct local_random_data unsafe_state =
  {
    .fptr = &randtbl[4],
    .rptr = &randtbl[1],
    .state = &randtbl[1],
    .rand_type = 3,
    .rand_deg = 31,
    .rand_sep = 3,
    .end_ptr = &randtbl[sizeof (randtbl) / sizeof (randtbl[0])]
};

static int local_random()
{
  int *state;
  int result;

  int *fptr = unsafe_state.fptr;
  int *rptr = unsafe_state.rptr;
  int *end_ptr = unsafe_state.end_ptr;
  int val;

  state = unsafe_state.state;

  val = *fptr += *rptr;
  /* Chucking least random bit.  */
  result = (val >> 1) & 0x7fffffff;
  ++fptr;
  if (fptr >= end_ptr)
    {
      fptr = state;
      ++rptr;
    }
  else
    {
      ++rptr;
      if (rptr >= end_ptr)
	rptr = state;
    }
  unsafe_state.fptr = fptr;
  unsafe_state.rptr = rptr;
  return result;
}

int local_srandom(unsigned int seed)
{
  int type;
  int *state;
  long int i;
  long int word;
  int *dst;
  int kc;

  type = unsafe_state.rand_type;

  state = unsafe_state.state;
  if (seed == 0)
    seed = 1;
  state[0] = seed;

  dst = state;
  word = seed;
  kc = unsafe_state.rand_deg;
  for (i = 1; i < kc; ++i)
    {
      long int hi = word / 127773;
      long int lo = word % 127773;
      word = 16807 * lo - 2836 * hi;
      if (word < 0)
	word += 2147483647;
      *++dst = word;
    }

  unsafe_state.fptr = &state[unsafe_state.rand_sep];
  unsafe_state.rptr = &state[0];
  kc *= 10;
  while (--kc >= 0)
    {
      (void) local_random();
    }

 done:
  return 0;

 fail:
  return -1;
}

static void permute(int n, int *values)
{
  int i,j,tmp;

  for(i=0;i<(n-1);i++)
  {
    j=local_random()%(n-i)+i;
    tmp=values[i];
    values[i]=values[j];
    values[j]=tmp;
  }
}

static int check(int r, int c, int n)
{
  int i,j;
  for(i=0;i<9;i++)
  { if(setboard[i][c]==n) return 0; if(setboard[r][i]==n) return 0; }
  for(i=0;i<3;i++) for(j=0;j<3;j++)
  { if(setboard[r/3*3+i][c/3*3+j]==n) return 0; }
  return 1;
}

static void findpossible()
{
  int i,j,i1,j1,p;

  for(i=0; i<9; i++)
  {
    for(j=0; j<9; j++)
    {
      if(board[i][j])
      {
	p=0;
      }
      else
      {
	p=1022;
	for(i1=0; i1<9; i1++)
	{
	  p &= ~(1<<board[i][i1]);
	  p &= ~(1<<board[i1][j]);
	}
	for(i1=0;i1<3;i1++)
	for(j1=0;j1<3;j1++)
	  p &= ~(1<<board[i/3*3+i1][j/3*3+j1]);
      }
      possible[i][j] = p;
    }
  }
}

static void printout_possible()
{
  int i,j,k,n,k1,p;
  for(i=0;i<9;i++)
  {
    for(j=0;j<9;j++)
    {
      if(board[i][j])
      {
	logprintf("   %d   ",board[i][j]);
      }
      else
      {
	p=possible[i][j];
	k=2;
	n=0;
	for(k1=1;k1<=9;k1++)
	{
	  if(p&k)
	  {
	    logprintf("%d",k1);
	    n++;
	  }
	  k = k<<1;
	}
	for(;n<7;n++) logprintf(" ");
      }
    }
    logprintf("\n");
  }
}

static int test_cell(int *nindices, int *indices, int *r, int *c)
{
  int best, bestp, i, j, p, n;
  best = 999;
  bestp = 0;
  *nindices=0;

  for(i=0; i<9; i++)
  {
    for(j=0; j<9; j++)
    {
      if(board[i][j]==0)
      {
	p=possible[i][j];
	n = NBITS(p);
	if(n<best)
	{
	  best=n;
	  bestp=p;
	  /* if n is zero, we've reached a dead end */
	  if(n==0)
	  {
	    *nindices=-1;
	    return;
	  }
	  *r = i;
	  *c = j;
	}
      }
    }
  }
  for(i=1;bestp;i++)
  {
    bestp >>= 1;
    if(1 & bestp)
    {
      indices[(*nindices)++]=i;
    }
  }
}

static int test_number(int *nindices, int *indices, int *r, int *c)
{
  int i,j,p,k,k1,n,l,l1,l2;
  for(i=0;i<9;i++)
  {
    for(j=0; j<9; j++)
    {
      if(p=possible[i][j])
      {
	k=2;
	for(k1=1;k1<=9;k1++)
	{
	  if(k&p)
	  {
	    n=0;
	    for(l=0;l<9;l++)
	    {
	      if(possible[i][l]&k)n++;
	    }
	    if(n==1)
	    {
	      if(printout) logprintf("%d for (%d,%d) only candidate in row %d.\n", k1, i+1, j+1, i+1);
	      *r=i;
	      *c=j;
	      indices[0]=k1;
	      *nindices=1;
	      return 1;
	    }
	    n=0;
	    for(l=0;l<9;l++)
	    {
	      if(possible[l][j]&k)n++;
	    }
	    if(n==1)
	    {
	      if(printout) logprintf("%d for (%d,%d) only candidate in column %d.\n", k1, i+1, j+1, j+1);
	      *r=i;
	      *c=j;
	      indices[0]=k1;
	      *nindices=1;
	      return 1;
	    }
	    n=0;
	    for(l1=0;l1<3;l1++)
	    for(l2=0;l2<3;l2++)
	    {
	      if(possible[i/3*3+l1][j/3*3+l2]&k)n++;
	    }
	    if(n==1)
	    {
	      if(printout) logprintf("%d for (%d,%d) only candidate in box [%d,%d].\n", k1, i+1, j+1, i/3+1, j/3+1);
	      *r=i;
	      *c=j;
	      indices[0]=k1;
	      *nindices=1;
	      return 1;
	    }
	  }
	  k = k<<1;
	}
      }
    }
  }
  return 0;
}

static int test_ssc()
{
  /*
  	Find single sector candidates
  */
  int i,i1,i2,j,j1,j2,k,k1,foundany,foundthispass,sector,noccur,isector,jsector;

  foundany=0;
  for(k=2,k1=1;k1<=9;k<<=1,k1++)
  {
    /* check rows i for occurance of k1's in a single sector */
    for(i=0;i<9;i++)
    {
      sector = -1;
      noccur = -1;
      for(j=0;j<9;j++)
      {
	if(possible[i][j]&k)
	{
	  if(sector == -1)
	  {
	    /* first occurance of k in row i: */
	    sector = j/3;
	    noccur = 1;
	  }
	  else if(sector == j/3)
	  {
	    /* occurs in same sector */
	    noccur++;
	  }
	  else
	  {
	    /* occurs in another sector; not candidate */
	    goto NOT_CANDIDATE1;
	  }
	}
      }
      if(noccur>1)
      {
	/* eliminating other occurences */
	isector=i/3*3;
	jsector=sector*3;
	for(i1=0;i1<3;i1++)
	{
	  if(isector+i1!=i)
	  {
	    for(i2=0;i2<3;i2++)
	    {
	      if(possible[isector+i1][jsector+i2]&k)
	      {
		if(printout) logprintf("Eliminating %d from (%d,%d) because of single sector occurance in row %d\n", k1, isector+i1+1, jsector+i2+1, i+1);
		possible[isector+i1][jsector+i2] &= ~k;
		foundany=1;
	      }
	    }
	  }
	}
	if(foundany) return 1;
      }
NOT_CANDIDATE1: ;
    }
    /* check columns j for occurance of k1's in a single sector */
    for(j=0;j<9;j++)
    {
      sector = -1;
      noccur = -1;
      for(i=0;i<9;i++)
      {
	if(possible[i][j]&k)
	{
	  if(sector == -1)
	  {
	    /* first occurance of k in column j: */
	    sector = i/3;
	    noccur = 1;
	  }
	  else if(sector == i/3)
	  {
	    /* occurs in same sector */
	    noccur++;
	  }
	  else
	  {
	    /* occurs in another sector; not candidate */
	    goto NOT_CANDIDATE2;
	  }
	}
      }
      if(noccur>1)
      {
	/* eliminating other occurences */
	isector=sector*3;
	jsector=j/3*3;
	for(i1=0;i1<3;i1++)
	{
	  for(i2=0;i2<3;i2++)
	  {
	    if(jsector+i2!=j)
	    {
	      if(possible[isector+i1][jsector+i2]&k)
	      {
		if(printout) logprintf("Eliminating %d from (%d,%d) because of single sector occurance in column %d\n", k1, isector+i1+1, jsector+i2+1, j+1);
		possible[isector+i1][jsector+i2] &= ~k;
		foundany=1;
	      }
	    }
	  }
	}
	if(foundany) return 1;
      }
NOT_CANDIDATE2: ;
    }
    for(i1=0;i1<3;i1++)
    for(i2=0;i2<3;i2++)
    {
      sector = -1;
      noccur = -1;
      for(j1=0;j1<3;j1++)
      for(j2=0;j2<3;j2++)
      {
	if(possible[i1*3+j1][i2*3+j2]&k)
	{
	  if(sector == -1)
	  {
	    /* first occurance of k in box[i1,j2]: */
	    sector = j1;
	    noccur = 1;
	  }
	  else if(sector == j1)
	  {
	    /* occurs in same row */
	    noccur++;
	  }
	  else
	  {
	    /* occurs in another row; not candidate */
	    goto NOT_CANDIDATE3;
	  }
	}
      }
      if(noccur>1)
      {
	/* eliminating other occurences */
	isector=sector+i1*3;
	for(j=0;j<9;j++)
	{
	  if(j/3 != i2)
	  {
	    if(possible[isector][j]&k)
	    {
	      if(printout) logprintf("Eliminating %d from (%d,%d) because of single row occurance in sector [%d,%d]\n", k1, isector+1, j+1, i1+1, i2+1);
	      possible[isector][j] &= ~k;
	      foundany=1;
	    }
	  }
	}
	if(foundany) return 1;
      }
NOT_CANDIDATE3: ;
    }
    for(i1=0;i1<3;i1++)
    for(i2=0;i2<3;i2++)
    {
      sector = -1;
      noccur = -1;
      for(j1=0;j1<3;j1++)
      for(j2=0;j2<3;j2++)
      {
	if(possible[i1*3+j1][i2*3+j2]&k)
	{
	  if(sector == -1)
	  {
	    /* first occurance of k in box[i1,j2]: */
	    sector = j2;
	    noccur = 1;
	  }
	  else if(sector == j2)
	  {
	    /* occurs in same column */
	    noccur++;
	  }
	  else
	  {
	    /* occurs in another column; not candidate */
	    goto NOT_CANDIDATE4;
	  }
	}
      }
      if(noccur>1)
      {
	/* eliminating other occurences */
	jsector=sector+i2*3;
	for(i=0;i<9;i++)
	{
	  if(i/3 != i1)
	  {
	    if(possible[i][jsector]&k)
	    {
	      if(printout) logprintf("Eliminating %d from (%d,%d) because of single column occurance in sector [%d,%d]\n", k1, i+1, jsector+1, i1+1, i2+1);
	      possible[i][jsector] &= ~k;
	      foundany=1;
	    }
	  }
	}
	if(foundany) return 1;
      }
NOT_CANDIDATE4: ;
    }
  }
  return 0;
}

static int bitmaps[502]={
6,10,12,18,20,24,34,36,40,48,
66,68,72,80,96,130,132,136,144,160,
192,258,260,264,272,288,320,384,514,516,
520,528,544,576,640,768,28,134,50,138,
140,52,146,148,152,56,162,164,168,176,
22,194,196,200,208,224,14,70,262,38,
266,268,74,274,276,280,76,290,292,296,
304,26,322,324,328,336,352,82,386,388,
392,400,416,448,84,88,518,42,522,524,
98,530,532,536,100,546,548,552,560,104,
578,580,584,592,608,112,642,644,648,656,
672,704,44,770,772,776,784,800,832,896,
184,330,332,86,338,340,344,60,354,356,
360,368,90,198,92,390,202,394,396,204,
402,404,408,30,418,420,424,432,210,450,
452,456,464,480,212,216,78,226,228,232,
526,240,142,54,534,102,538,540,46,150,
106,550,270,554,556,154,562,564,568,156,
108,278,582,58,586,588,282,594,596,600,
284,610,612,616,624,114,166,116,646,294,
650,652,170,658,660,664,298,674,676,680,
688,300,706,708,712,720,736,172,306,308,
774,312,778,780,120,786,788,792,178,802,
804,808,816,180,834,836,840,848,864,326,
898,900,904,912,928,960,188,590,422,218,
426,598,428,602,604,220,434,436,614,440,
618,620,62,626,628,632,334,110,454,158,
458,460,230,654,466,468,472,662,342,666,
668,482,484,488,678,496,682,684,286,690,
692,696,346,348,174,710,234,714,716,236,
722,724,728,358,738,740,744,752,118,362,
364,242,370,372,376,782,542,244,248,790,
302,794,796,94,122,206,806,310,810,812,
558,818,820,824,398,182,314,838,566,842,
844,316,850,852,856,570,866,868,872,880,
572,406,124,902,410,906,908,412,914,916,
920,186,930,932,936,944,214,962,964,968,
976,992,798,686,486,438,490,694,492,698,
814,700,622,380,822,498,826,828,500,630,
504,634,718,636,442,846,444,726,574,854,
730,858,860,732,246,238,870,414,874,876,
742,882,884,888,250,746,748,252,754,756,
760,910,126,366,462,918,222,922,924,350,
318,470,934,670,938,940,430,946,948,952,
474,476,606,966,374,970,972,190,978,980,
984,378,994,996,1000,1008,892,758,942,734,
762,764,950,506,954,956,862,830,508,494,
478,702,382,974,254,878,750,982,926,986,
988,502,446,886,998,638,1002,1004,890,1010,
1012,1016,510,1006,894,766,990,1014,958,1018,
1020,0};

static int test_ds_help(int type, int r, int c)
{
  int i,j,k,k1,nmatch,nor,nbits,ibitmap,bitmap,elim,anyeliminated;
  int slice[9];
  char buf1[512], buf2[512];
  int printed;

  anyeliminated=0;

  if(type==0)
  {
    for(j=0;j<9;j++) slice[j]=possible[r][j];
  }
  else if(type==1)
  {
    for(i=0;i<9;i++) slice[i]=possible[i][c];
  }
  else
  {
    for(i=0;i<3;i++)
    for(j=0;j<3;j++) slice[i*3+j]=possible[r*3+i][c*3+j];
  }

  for(ibitmap=0; bitmaps[ibitmap] && !anyeliminated; ibitmap++)
  {
    bitmap=bitmaps[ibitmap];
    nbits=NBITS(bitmap);
    nmatch=0;
    nor=0;
    for(i=0;i<9;i++)
    {
      if(slice[i]&bitmap)
      {
        nmatch++;
	nor |= (slice[i]&bitmap);
      }
    }
    if(nmatch==nbits && nor==bitmap)
    {
      if(printout)
      {
	sprintf(buf1, "The values ");
	for(k=2,k1=1;k1<=9;k<<=1,k1++)
	{
	  if(k&bitmap)
	  {
	    sprintf(buf2, " %d", k1, k);
	    strcat(buf1, buf2);
	  }
	}
	strcat(buf1, " occupy the cells");
	for(i=0;i<9;i++)
	{
	  if(slice[i]&bitmap)
	  {
	    if(type==0) sprintf(buf2, " (%d,%d)", r+1, i+1);
	    else if(type==1) sprintf(buf2, " (%d,%d)", i+1, c+1);
	    else sprintf(buf2, " (%d,%d)", r*3+i/3+1, c*3+i%3+1);
	    strcat(buf1, buf2);
	  }
	}
	strcat(buf1, " in some order.\n");
	printed=0;
      }
      for(i=0;i<9;i++)
      {
	if(slice[i]&bitmap)
	{
	  elim=slice[i]&(~bitmap);
	  if(elim)
	  {
	    anyeliminated=1;
	    if(printout)
	    {
	      for(k=2,k1=1;k1<=9;k<<=1,k1++)
	      {
		if(k&elim)
		{
		  if(!printed)
		  {
		    logprintf("%s", buf1);
		    printed=1;
		  }
		  logprintf("- eliminating %d from", k1);
		  if(type==0) logprintf(" (%d,%d)\n", r+1, i+1);
		  else if(type==1) logprintf(" (%d,%d)\n", i+1, c+1);
		  else logprintf(" (%d,%d)\n", r*3+i/3+1, c*3+i%3+1);
		}
	      }
	    }
	    slice[i] &= bitmap;
	  }
	}
      }
    }
  }
  if(anyeliminated)
  {
    if(type==0)
    {
      for(j=0;j<9;j++) possible[r][j]=slice[j];
    }
    else if(type==1)
    {
      for(i=0;i<9;i++) possible[i][c]=slice[i];
    }
    else
    {
      for(i=0;i<3;i++)
      for(j=0;j<3;j++) possible[r*3+i][c*3+j]=slice[i*3+j];
    }
  }
  return anyeliminated;
}

static int test_ds()
{
  int i,j;

  for(i=0;i<9;i++) if(test_ds_help(0, i, 0)) goto ELIMINATED;
  for(j=0;j<9;j++) if(test_ds_help(1, 0, j)) goto ELIMINATED;
  for(i=0;i<3;i++)
  for(j=0;j<3;j++) if(test_ds_help(2, i, j)) goto ELIMINATED;
  return 0;
ELIMINATED:
  return 1;
}

static int test_guess(int *nindices, int *indices, int r, int c)
{
  int iindex,i,j,saveprintout,n,savensol;
  int localpossible[9][9];

  saveprintout=printout;
  savensol=nsol;
  printout=0;

  for(i=0;i<9;i++)
  {
    for(j=0;j<9;j++)
    localpossible[i][j]=possible[i][j];
  }
  for(iindex=0; iindex<*nindices; iindex++)
  {
    board[r][c]=indices[iindex];
    n=~(1<<indices[iindex]);
    for(i=0;i<9;i++)
    {
      possible[r][i] &= n;
      possible[i][c] &= n;
    }
    for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      possible[r/3*3+i][c/3*3+j] &= n;
    possible[r][c]=0;
    nsol=0;
    move();
    if(nsol==1)
    {
      *nindices = 1;
      indices[0] = indices[iindex];
      printout = saveprintout;
      nsol = savensol;
      return 1;
    }
    board[r][c]=0;
    for(i=0;i<9;i++)
    {
      for(j=0;j<9;j++)
      possible[i][j]=localpossible[i][j];
    }
  }
  printout = saveprintout;
  nsol = savensol;
  if(printout) logprintf("Can't find any solutions...\n");
  return 0;
}

static void move()
{
  int r,c,i,j,p,k,n,k1,i1,j1;
  int indices[10];
  int nindices;
  int localpossible[9][9];
  int saveprintout;
  int anyeliminated,eliminatedthis;
  nmove++;
  r = -1;
  c = -1;
  test_cell(&nindices, indices, &r, &c);
  /* if nindices is -1, we've reached a dead end */
  if(nindices == -1) return;
  /* if r is -1, all cells are filled, and we're done */
  if(r== -1)
  {
    nsol++;
    return;
  }
  if(nindices>1 && fastalg==0)
  {
    if(test_number(&nindices, indices, &r, &c))
    {
      nscnumber++;
      goto FOUND2;
    }
    while(test_ssc())
    {
      /* eliminated ssc, call test_cell and test_number again */ 
      test_cell(&nindices, indices, &r, &c);
      if(nindices==1)
      {
        if(printout) logprintf("The SSC elimination gave us a unique candidate: %d in (%d,%d)\n", indices[0], r+1, c+1);
        nssc++;
	goto FOUND2;
      }
      if(test_number(&nindices, indices, &r, &c))
      {
        if(printout) logprintf("The SSC elimination gave us a unique candidate: %d in (%d,%d)\n", indices[0], r+1, c+1);
        nssc++;
	goto FOUND2;
      }
    }
    while(test_ds())
    {
      /* eliminated ds, call test_cell and test_number again */ 
      test_cell(&nindices, indices, &r, &c);
      if(nindices==1)
      {
        if(printout) logprintf("The Disjoint Subset elimination gave us a unique candidate: %d in (%d,%d)\n", indices[0], r+1, c+1);
        nds++;
	goto FOUND2;
      }
      if(test_number(&nindices, indices, &r, &c))
      {
        if(printout) logprintf("The Disjoint Subset elimination gave us a unique candidate: %d in (%d,%d)\n", indices[0], r+1, c+1);
        nds++;
	goto FOUND2;
      }
    }
    if(printout)
    {
      logprintf("Running out of algorithms; I'll have to guess - trying");
      for(i=0;i<nindices;i++) logprintf(" %d", indices[i]);
      logprintf(" for (%d,%d): ",r+1,c+1);
    }
    if(test_guess(&nindices, indices, r, c))
    {
      if(printout) logprintf("%d is OK.\n", indices[0]);
      nmultiple++;
      goto FOUND2;
    }
  }
  else if(nindices==1)
  {
    nsccell++;
  }
  if(printout)
  {
    if(nindices==1)
    {
      logprintf("Only one candidate in (%d,%d): %d\n", r+1, c+1, indices[0]);
    }
    else
    {
      logprintf("%d candidates in (%d,%d):", nindices, r+1, c+1);
      for(i=0;i<nindices;i++) logprintf(" %d", indices[i]);
      logprintf("\n");
    }
  }
  if(randomize) permute(nindices, indices);
FOUND2:
  for(i=0;i<9;i++)
  {
    for(j=0;j<9;j++)
    localpossible[i][j]=possible[i][j];
  }
  for(i=0;i<nindices;i++)
  {
    board[r][c]=indices[i];
    n=~(1<<indices[i]);
    for(i1=0;i1<9;i1++)
    {
      possible[r][i1] &= n;
      possible[i1][c] &= n;
    }
    for(i1=0;i1<3;i1++)
    for(j1=0;j1<3;j1++)
      possible[r/3*3+i1][c/3*3+j1] &= n;
    possible[r][c]=0;
    move();
    if(nsol>1) return;
    board[r][c]=0;
    for(i1=0;i1<9;i1++)
    {
      for(j1=0;j1<9;j1++)
      possible[i1][j1]=localpossible[i1][j1];
    }
  }
}

static int testdigits()
{
  int i,j,i1,j1;
  int savefastalg;
  char windowtitle[80];

  for(i1=0; i1<9; i1++)
  for(j1=0; j1<9; j1++)
  board[i1][j1] = setboard[i1][j1];
  nmultiple=0;
  nsccell=0;
  nscnumber=0;
  nssc=0;
  nds=0;
  savefastalg=fastalg;
  fastalg=0;
  findpossible();
  move();
  fastalg=savefastalg;
  if(level==0 && (nmultiple>0 || nds>0 || nssc>0 || nscnumber>0)) return 0;
  if(level==1 && (nmultiple>0 || nds>0 || nssc>0 || nscnumber==0)) return 0;
  if(level==2 && (nmultiple>0 || nds>0 || nssc==0)) return 0;
  if(level==3 && (nmultiple>0 || nds==0)) return 0;
  if(level==4 && nmultiple==0) return 0;

  if(ndigits<=bestndigits)
  {
    for(i=0; i<9; i++)
    for(j=0; j<9; j++)
    displayboard[i][j] = setboard[i][j];
    bestndigits=ndigits;
    sprintf(windowtitle, "Sudoku Generator. %d cells in %d iteration%s", ndigits, ntest, ntest==1?"":"s");
    settitle(windowtitle);
    main_update();
    if(bestndigits<=nfilled)
    {
      running=0;
      setcreatemenutext("Create");
      return 1;
    }
  }
  return 0;
}

void sudoku_run()
{
  int i,j,i1,j1;
  int i2,j2,k2;
  int walktype;
  int indices[81],*pindex,nindices;

  solved=0;
  fastalg=1;

  ntest++;

  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  board[i][j] = 0;

  randomize=1;

  nsol=0;
  findpossible();
  move();
  if(nsol==0)
  {
    fprintf(stderr, "Hm, can't happen.\n");
    exit(0);
  }

  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  setboardsave[i][j] = board[i][j];

  randomize=0;

  pindex=indices;
  if(symmetrical)
  {
    nindices=25;
    for(i=0;i<5;i++)
    for(j=0;j<5;j++)
    {
      *pindex++ = (i<<4)|j;
    }
  }
  else
  {
    nindices=81;
    for(i=0;i<9;i++)
    for(j=0;j<9;j++)
    {
      *pindex++ = (i<<4)|j;
    }
  }
  permute(nindices,indices);
  for(i1=0; i1<9; i1++)
  for(j1=0; j1<9; j1++)
  setboard[i1][j1] = setboardsave[i1][j1];
  ndigits=81;
  for(i2=0;i2<nindices;i2++)
  {
    i = indices[i2]>>4;
    j = indices[i2]&15;
    for(i1=0; i1<9; i1++)
    for(j1=0; j1<9; j1++)
    board[i1][j1] = setboard[i1][j1];
    if(symmetrical)
    {
      board[4+i][4+j] = 0;
      board[4+i][4-j] = 0;
      board[4-i][4+j] = 0;
      board[4-i][4-j] = 0;
    }
    else
    {
      board[i][j] = 0;
    }
    nsol=0;
    findpossible();
    move();
    if(nsol==1)
    {
      if(symmetrical)
      {
	setboard[4+i][4+j] = 0;
	setboard[4+i][4-j] = 0;
	setboard[4-i][4+j] = 0;
	setboard[4-i][4-j] = 0;
	if((i==0)&&(j==0))
	{
	  ndigits -= 1;
	}
	else if((i==0)||(j==0))
	{
	  ndigits -= 2;
	}
	else
	{
	  ndigits -= 4;
	}
      }
      else
      {
	setboard[i][j] = 0;
	ndigits--;
      }
      if(ndigits<=nfilled) break;
    }
  }
  if(testdigits()) return;
}

void unsolve_sudoku()
{
  int i,j;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  displayboard[i][j] = beforeboard[i][j];
  solved=0;
}

int solve_sudoku(int newprintout)
{
  int i,j;
  fastalg=1;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  {
    board[i][j] = displayboard[i][j];
    beforeboard[i][j] = displayboard[i][j];
  }
  randomize=0;
  nsol=0;
  findpossible();
  move();
  if(nsol!=1)
  {
    if(nsol>1)
    {
      for(i=0; i<9; i++)
      for(j=0; j<9; j++)
      displayboard[i][j] = beforeboard[i][j]?board[i][j]:-board[i][j];
      solved=1;
    }
    return nsol;
  }

  fastalg=0;
  for(i=0; i<9; i++)
  for(j=0; j<9; j++)
  {
    board[i][j] = displayboard[i][j];
    beforeboard[i][j] = displayboard[i][j];
  }
  randomize=0;
  nsol=1;
  findpossible();
  printout=newprintout;
  logprintf("Solution:\n");
  /* printout_possible(); */
  move();
  logprintf("- done!\n");
  printout=0;
  if(nsol>0)
  {
    for(i=0; i<9; i++)
    for(j=0; j<9; j++)
    displayboard[i][j] = beforeboard[i][j]?board[i][j]:-board[i][j];
    solved=1;
  }
  return 1;
}

